<?php
/**
*
* @package antibot100500
* @copyright (c) 2012,2014 Nekstati http://phpbbguru.net & c61 c61@yandex.ru http://c61.su
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/
namespace c61\antibot100500\event;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
* Event listener
*/
class listener implements EventSubscriberInterface
{	
	/** @var \phpbb\auth\auth */
	protected $auth;
	
	/** @var \phpbb\config\config */
	protected $config;
	
	/** @var \phpbb\template\template */
	protected $template;
	
	/** @var \phpbb\user */
	protected $user;
	
	/** @var \phpbb\db\driver\driver_interface */
	protected $db;
	
	/** @var string */
	protected $phpbb_root_path;
	protected $php_ext;
	
	/**
	* Constructor
	* 
	* @param \phpbb\auth\auth $auth
	* @param \phpbb\config\config $config
	* @param \phpbb\template\template $template
	* @param \phpbb\user $user
	* @param \phpbb\db\driver\driver_interface $db
	* @param string $phpbb_root_path Root path
	* @param string $php_ext
	*/
	public function __construct(\phpbb\auth\auth $auth, \phpbb\config\config $config, \phpbb\template\template $template, \phpbb\user $user, \phpbb\db\driver\driver_interface $db, $phpbb_root_path, $php_ext)
	{
		$this->auth = $auth;
		$this->config = $config;
		$this->template = $template;
		$this->user = $user;
		$this->db = $db;
		$this->phpbb_root_path = $phpbb_root_path;
		$this->php_ext = $php_ext;
	}
	
	/**
	* Assign functions defined in this class to event listeners in the core
	*
	* @return array
	* @static
	* @access public
	*/
	static public function getSubscribedEvents()
	{
		return array(
			/**
			* Event to load language files and modify user data on every page
			*
			* @event core.user_setup
			* @var	array	user_data			Array with user's data row
			* @var	string	user_lang_name		Basename of the user's langauge
			* @var	string	user_date_format	User's date/time format
			* @var	string	user_timezone		User's timezone, should be one of
			*							http://www.php.net/manual/en/timezones.php
			* @var	mixed	lang_set			String or array of language files
			* @var	array	lang_set_ext		Array containing entries of format
			* 					array(
			* 						'ext_name' => (string) [extension name],
			* 						'lang_set' => (string|array) [language files],
			* 					)
			* 					For performance reasons, only load translations
			* 					that are absolutely needed globally using this
			* 					event. Use local events otherwise.
			* @var	mixed	style_id			Style we are going to display
			* @since 3.1.0-a1
			*/
			'core.user_setup'			=> 'lang_antibot100500',

			/**
			* Perform additional actions on creation of the form token
			*
			* @event core.add_form_key
			* @var	string	form_name			The form name
			* @var	int		now					Current time timestamp
			* @var	string	s_fields			Generated hidden fields
			* @var	string	token				Form token
			* @var	string	token_sid			User session ID
			*
			* @since 3.1.0-RC3
			*/
			'core.add_form_key'			=> 'event_antibot100500',

			/**
			* Execute code and/or modify output before displaying the template.
			*
			* @event core.page_footer_after
			* @var	bool display_template	Whether or not to display the template
			* @var	bool exit_handler		Whether or not to run the exit_handler()
			*
			* @since 3.1.0-RC5
			*/
			'core.page_footer_after'	=> 'log_antobot100500',
			'core.permissions'			=> 'add_permission',
		);
	}
	
	// ON/OFF
	private $onoff_antibot = false;
	// ON/OFF for registered users
	private $onoff_registered = false;
	// Options
	private $onoff_encode = false;
	private $onoff_behavior = false;
	private $onoff_log_users = false;

	// Mobile browser detected
	private $mobile_browser = false;

	/** Encode with key(xor) **/
	private	$key = 85;	// DO NOT CHANGE VALUE !!!

	private function encode ( $in, $key ) {

		$out = '';
		for ( $i = 0; $i < strlen($in); $i++, $out .= $i != strlen($in) ? ';' : '' ) {
			$out .= strval(ord(substr($in,$i,1)) ^ $key);
		}
		return $out;
	}

	/**
	* Events
	*/
	public function lang_antibot100500($event)
	{
		$lang_set_ext = $event['lang_set_ext'];
		$lang_set_ext[] = array(
			'ext_name' => 'c61/antibot100500',
			'lang_set' => 'antibot100500',
		);
		$event['lang_set_ext'] = $lang_set_ext;
	}

	public function event_antibot100500($event) {

		// NOT for adm ot install
		if ( defined('ADMIN_START') || defined('IN_INSTALL') ) return;

		// All vars OK ?
		if ( !isset($event['s_fields']) ) return;
		if ( !isset($event['token']) ) return;
		if ( !isset($event['now']) ) return;

		// ON/OFF
		if ( isset($this->config['antibot100500_onoff_antibot']) ) {
			$this->onoff_antibot = $this->config['antibot100500_onoff_antibot'] ? true : false;
		}
		// Switched OFF ?
		if ( !$this->onoff_antibot ) return;

		// ON/OFF for registered users
		if ( isset($this->config['antibot100500_onoff_registered']) ) {
			$this->onoff_registered = $this->config['antibot100500_onoff_registered'] ? true : false;
		}
		// Registered user and OFF for registered users ?
		if ( $this->user->data['is_registered'] && !$this->onoff_registered ) return;

		// Options for paranoids
		if ( isset($this->config['antibot100500_onoff_encode']) ) {
			$this->onoff_encode = $this->config['antibot100500_onoff_encode'] ? true : false;
		}
		if ( isset($this->config['antibot100500_onoff_behavior']) ) {
			$this->onoff_behavior = $this->config['antibot100500_onoff_behavior'] ? true : false;
		}

		// Add "onsubmit" event (behavios analisis) for mobile browsers
		// NOTE: NOT compatible with Motorola Internet Browser
//
// see: http://detectmobilebrowsers.com/
//
		if ( $this->onoff_behavior ) {

			global $request;

			$useragent = $request->server('HTTP_USER_AGENT');
			if ( !empty($useragent) ) {
	       		if(preg_match('/(android|bb\d+|meego).+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\.(browser|link)|vodafone|wap|windows ce|xda|xiino/i',$useragent)||preg_match('/1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw\-(n|u)|c55\/|capi|ccwa|cdm\-|cell|chtm|cldc|cmd\-|co(mp|nd)|craw|da(it|ll|ng)|dbte|dc\-s|devi|dica|dmob|do(c|p)o|ds(12|\-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(\-|_)|g1 u|g560|gene|gf\-5|g\-mo|go(\.w|od)|gr(ad|un)|haie|hcit|hd\-(m|p|t)|hei\-|hi(pt|ta)|hp( i|ip)|hs\-c|ht(c(\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i\-(20|go|ma)|i230|iac( |\-|\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\/)|klon|kpt |kwc\-|kyo(c|k)|le(no|xi)|lg( g|\/(k|l|u)|50|54|\-[a-w])|libw|lynx|m1\-w|m3ga|m50\/|ma(te|ui|xo)|mc(01|21|ca)|m\-cr|me(rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi|de|do|t(\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)\-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|pg(13|\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\-2|po(ck|rt|se)|prox|psio|pt\-g|qa\-a|qc(07|12|21|32|60|\-[2-7]|i\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\-|oo|p\-)|sdk\/|se(c(\-|0|1)|47|mc|nd|ri)|sgh\-|shar|sie(\-|m)|sk\-0|sl(45|id)|sm(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h\-|v\-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl\-|tdg\-|tel(i|m)|tim\-|t\-mo|to(pl|sh)|ts(70|m\-|m3|m5)|tx\-9|up(\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40|5[0-3]|\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(\-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|yas\-|your|zeto|zte\-/i',substr($useragent,0,4))) {
					$this->mobile_browser = true;
				}
			}
		}

		// form_token framing
		$subprev = rand(28,32);
		$subsucc = 60 - $subprev;
		$prev = substr(md5(dechex(rand(1,2147483647))),0,$subprev);
		$succ = substr(md5($this->user->session_id),0,$subsucc);

		// input id
		$form_tokens_name = 'form_tokens';

		// Unframing
		$script_src = '
	dec_'.$form_tokens_name.' = function() {
		for ( var i=0; i<'.$form_tokens_name.'.length; i++ ) {
			'.$form_tokens_name.'[i].value = '.$form_tokens_name.'[i].value.replace(/' . $prev . '|' . $succ . '/g, \'\');
		}
	}
';

		// Start scripts
		$code = '
<script type="text/javascript">// <![CDATA[
';

		// Input element
		$code .= '
	'.$form_tokens_name.' = document.getElementsByName("form_token");
';


//
// see: http://javascript.ru/tutorial/events/crossbrowser
// see: http://javascriptobfuscator.com/
//
		// Script for keyboard and mouse events precessing
		if ( $this->onoff_behavior ) $code .= !$this->onoff_encode ? '
	if ( typeof form_token_Event == "undefined" ) form_token_Event = (function() {
		var guid = 0
		function fixEvent(event) {
			event = event || window.event
			if ( event.isFixed ) {
				return event
			}
			event.isFixed = true 
			event.preventDefault = event.preventDefault || function(){this.returnValue = false}
			event.stopPropagation = event.stopPropagaton || function(){this.cancelBubble = true}
			if (!event.target) {
				event.target = event.srcElement
			}
			if (!event.relatedTarget && event.fromElement) {
				event.relatedTarget = event.fromElement == event.target ? event.toElement : event.fromElement;
			}
			if ( event.pageX == null && event.clientX != null ) {
				var html = document.documentElement, body = document.body;
				event.pageX = event.clientX + (html && html.scrollLeft || body && body.scrollLeft || 0) - (html.clientLeft || 0);
				event.pageY = event.clientY + (html && html.scrollTop || body && body.scrollTop || 0) - (html.clientTop || 0);
			}
			if ( !event.which && event.button ) {
				event.which = (event.button & 1 ? 1 : ( event.button & 2 ? 3 : ( event.button & 4 ? 2 : 0 ) ));
			}
			return event
		}  
		function commonHandle(event) {
			event = fixEvent(event)
			var handlers = this.events[event.type]
			for ( var g in handlers ) {
				var handler = handlers[g]
				var ret = handler.call(this, event)
				if ( ret === false ) {
					event.preventDefault()
					event.stopPropagation()
				}
			}
		}
		return {
			add: function(elem, type, handler) {
				if (elem.setInterval && ( elem != window && !elem.frameElement ) ) {
					elem = window;
				}
				if (!handler.guid) {
					handler.guid = ++guid
				}
				if (!elem.events) {
					elem.events = {}
					elem.handle = function(event) {
						if (typeof Event !== "undefined") {
							return commonHandle.call(elem, event)
						}
					}
				}
				if (!elem.events[type]) {
					elem.events[type] = {}        
					if (elem.addEventListener)
						elem.addEventListener(type, elem.handle, false)
					else if (elem.attachEvent)
						elem.attachEvent("on" + type, elem.handle)
				}
				elem.events[type][handler.guid] = handler
			},
			remove: function(elem, type, handler) {
				var handlers = elem.events && elem.events[type]
				if (!handlers) return
				delete handlers[handler.guid]
				for(var any in handlers) return 
				if (elem.removeEventListener)
					elem.removeEventListener(type, elem.handle, false)
				else if (elem.detachEvent)
					elem.detachEvent("on" + type, elem.handle)
				delete elem.events[type]
				for (var any in elem.events) return
				try {
					delete elem.handle
					delete elem.events 
				} catch(e) { // IE
					elem.removeAttribute("handle")
					elem.removeAttribute("events")
				}
			} 
		}
	}())

' : '
	var _0x1442=["\x75\x6E\x64\x65\x66\x69\x6E\x65\x64","\x65\x76\x65\x6E\x74","\x69\x73\x46\x69\x78\x65\x64","\x70\x72\x65\x76\x65\x6E\x74\x44\x65\x66\x61\x75\x6C\x74","\x72\x65\x74\x75\x72\x6E\x56\x61\x6C\x75\x65","\x73\x74\x6F\x70\x50\x72\x6F\x70\x61\x67\x61\x74\x69\x6F\x6E","\x73\x74\x6F\x70\x50\x72\x6F\x70\x61\x67\x61\x74\x6F\x6E","\x63\x61\x6E\x63\x65\x6C\x42\x75\x62\x62\x6C\x65","\x74\x61\x72\x67\x65\x74","\x73\x72\x63\x45\x6C\x65\x6D\x65\x6E\x74","\x72\x65\x6C\x61\x74\x65\x64\x54\x61\x72\x67\x65\x74","\x66\x72\x6F\x6D\x45\x6C\x65\x6D\x65\x6E\x74","\x74\x6F\x45\x6C\x65\x6D\x65\x6E\x74","\x70\x61\x67\x65\x58","\x63\x6C\x69\x65\x6E\x74\x58","\x64\x6F\x63\x75\x6D\x65\x6E\x74\x45\x6C\x65\x6D\x65\x6E\x74","\x62\x6F\x64\x79","\x73\x63\x72\x6F\x6C\x6C\x4C\x65\x66\x74","\x63\x6C\x69\x65\x6E\x74\x4C\x65\x66\x74","\x70\x61\x67\x65\x59","\x63\x6C\x69\x65\x6E\x74\x59","\x73\x63\x72\x6F\x6C\x6C\x54\x6F\x70","\x63\x6C\x69\x65\x6E\x74\x54\x6F\x70","\x77\x68\x69\x63\x68","\x62\x75\x74\x74\x6F\x6E","\x74\x79\x70\x65","\x65\x76\x65\x6E\x74\x73","\x63\x61\x6C\x6C","\x73\x65\x74\x49\x6E\x74\x65\x72\x76\x61\x6C","\x66\x72\x61\x6D\x65\x45\x6C\x65\x6D\x65\x6E\x74","\x67\x75\x69\x64","\x68\x61\x6E\x64\x6C\x65","\x61\x64\x64\x45\x76\x65\x6E\x74\x4C\x69\x73\x74\x65\x6E\x65\x72","\x61\x74\x74\x61\x63\x68\x45\x76\x65\x6E\x74","\x6F\x6E","\x72\x65\x6D\x6F\x76\x65\x45\x76\x65\x6E\x74\x4C\x69\x73\x74\x65\x6E\x65\x72","\x64\x65\x74\x61\x63\x68\x45\x76\x65\x6E\x74","\x72\x65\x6D\x6F\x76\x65\x41\x74\x74\x72\x69\x62\x75\x74\x65"];if( typeof form_token_Event==_0x1442[0]){form_token_Event=(function (){var _0x4049x1=0;function _0x4049x2(_0x4049x3){_0x4049x3=_0x4049x3||window[_0x1442[1]];if(_0x4049x3[_0x1442[2]]){return _0x4049x3;} ;_0x4049x3[_0x1442[2]]=true;_0x4049x3[_0x1442[3]]=_0x4049x3[_0x1442[3]]||function (){this[_0x1442[4]]=false;} ;_0x4049x3[_0x1442[5]]=_0x4049x3[_0x1442[6]]||function (){this[_0x1442[7]]=true;} ;if(!_0x4049x3[_0x1442[8]]){_0x4049x3[_0x1442[8]]=_0x4049x3[_0x1442[9]];} ;if(!_0x4049x3[_0x1442[10]]&&_0x4049x3[_0x1442[11]]){_0x4049x3[_0x1442[10]]=_0x4049x3[_0x1442[11]]==_0x4049x3[_0x1442[8]]?_0x4049x3[_0x1442[12]]:_0x4049x3[_0x1442[11]];} ;if(_0x4049x3[_0x1442[13]]==null&&_0x4049x3[_0x1442[14]]!=null){var _0x4049x4=document[_0x1442[15]],_0x4049x5=document[_0x1442[16]];_0x4049x3[_0x1442[13]]=_0x4049x3[_0x1442[14]]+(_0x4049x4&&_0x4049x4[_0x1442[17]]||_0x4049x5&&_0x4049x5[_0x1442[17]]||0)-(_0x4049x4[_0x1442[18]]||0);_0x4049x3[_0x1442[19]]=_0x4049x3[_0x1442[20]]+(_0x4049x4&&_0x4049x4[_0x1442[21]]||_0x4049x5&&_0x4049x5[_0x1442[21]]||0)-(_0x4049x4[_0x1442[22]]||0);} ;if(!_0x4049x3[_0x1442[23]]&&_0x4049x3[_0x1442[24]]){_0x4049x3[_0x1442[23]]=(_0x4049x3[_0x1442[24]]&1?1:(_0x4049x3[_0x1442[24]]&2?3:(_0x4049x3[_0x1442[24]]&4?2:0)));} ;return _0x4049x3;} ;function _0x4049x6(_0x4049x3){_0x4049x3=_0x4049x2(_0x4049x3);var _0x4049x7=this[_0x1442[26]][_0x4049x3[_0x1442[25]]];for(var _0x4049x8 in _0x4049x7){var _0x4049x9=_0x4049x7[_0x4049x8];var _0x4049xa=_0x4049x9[_0x1442[27]](this,_0x4049x3);if(_0x4049xa===false){_0x4049x3[_0x1442[3]]();_0x4049x3[_0x1442[5]]();} ;} ;} ;return {add:function (_0x4049xb,_0x4049xc,_0x4049x9){if(_0x4049xb[_0x1442[28]]&&(_0x4049xb!=window&&!_0x4049xb[_0x1442[29]])){_0x4049xb=window;} ;if(!_0x4049x9[_0x1442[30]]){_0x4049x9[_0x1442[30]]=++_0x4049x1;} ;if(!_0x4049xb[_0x1442[26]]){_0x4049xb[_0x1442[26]]={};_0x4049xb[_0x1442[31]]=function (_0x4049x3){if( typeof Event!==_0x1442[0]){return _0x4049x6[_0x1442[27]](_0x4049xb,_0x4049x3);} ;} ;} ;if(!_0x4049xb[_0x1442[26]][_0x4049xc]){_0x4049xb[_0x1442[26]][_0x4049xc]={};if(_0x4049xb[_0x1442[32]]){_0x4049xb[_0x1442[32]](_0x4049xc,_0x4049xb[_0x1442[31]],false);} else {if(_0x4049xb[_0x1442[33]]){_0x4049xb[_0x1442[33]](_0x1442[34]+_0x4049xc,_0x4049xb[_0x1442[31]]);} ;} ;} ;_0x4049xb[_0x1442[26]][_0x4049xc][_0x4049x9[_0x1442[30]]]=_0x4049x9;} ,remove:function (_0x4049xb,_0x4049xc,_0x4049x9){var _0x4049x7=_0x4049xb[_0x1442[26]]&&_0x4049xb[_0x1442[26]][_0x4049xc];if(!_0x4049x7){return ;} ;delete _0x4049x7[_0x4049x9[_0x1442[30]]];for(var _0x4049xd in _0x4049x7){return ;} ;if(_0x4049xb[_0x1442[35]]){_0x4049xb[_0x1442[35]](_0x4049xc,_0x4049xb[_0x1442[31]],false);} else {if(_0x4049xb[_0x1442[36]]){_0x4049xb[_0x1442[36]](_0x1442[34]+_0x4049xc,_0x4049xb[_0x1442[31]]);} ;} ;delete _0x4049xb[_0x1442[26]][_0x4049xc];for(var _0x4049xd in _0x4049xb[_0x1442[26]]){return ;} ;try{delete _0x4049xb[_0x1442[31]];delete _0x4049xb[_0x1442[26]];} catch(e){_0x4049xb[_0x1442[37]](_0x1442[31]);_0x4049xb[_0x1442[37]](_0x1442[26]);} ;} };} ());} ;
';

//
// see: http://javascriptobfuscator.com/
//
		// Original script
/*
		for ( var x = s.split(';'), i=0, z=''; i<x.length; i++) z += String.fromCharCode(x[i]^85);
		var js = document.createElement('SCRIPT');
		js.text = z;
		document.documentElement.children[0].appendChild(js);
*/
		// Obfuscated decoder script
		if ( $this->onoff_encode ) $code .= '
	func_'.$form_tokens_name.' = function(s) {
		var _0x829a=["\x3B","\x73\x70\x6C\x69\x74","","\x6C\x65\x6E\x67\x74\x68","\x66\x72\x6F\x6D\x43\x68\x61\x72\x43\x6F\x64\x65","\x53\x43\x52\x49\x50\x54","\x63\x72\x65\x61\x74\x65\x45\x6C\x65\x6D\x65\x6E\x74","\x74\x65\x78\x74","\x61\x70\x70\x65\x6E\x64\x43\x68\x69\x6C\x64","\x63\x68\x69\x6C\x64\x72\x65\x6E","\x64\x6F\x63\x75\x6D\x65\x6E\x74\x45\x6C\x65\x6D\x65\x6E\x74"];for(var x=s[_0x829a[1]](_0x829a[0]),i=0,z=_0x829a[2];i<x[_0x829a[3]];i++){z+=String[_0x829a[4]](x[i]^85);} ;var js=document[_0x829a[6]](_0x829a[5]);js[_0x829a[7]]=z;document[_0x829a[10]][_0x829a[9]][0][_0x829a[8]](js);
	}
';

		// Events handling and decoding
		if ( $this->onoff_behavior ) $code .= '
	handle_'.$form_tokens_name.' = function(event) {
		for ( var i=0; i<'.$form_tokens_name.'.length; i++ ) {
			form_token_Event.remove('.$form_tokens_name.'[i].form,\'keydown\',handle_'.$form_tokens_name.');
			form_token_Event.remove('.$form_tokens_name.'[i].form,\'mousedown\',handle_'.$form_tokens_name.');'
.($this->mobile_browser ?'
			form_token_Event.remove('.$form_tokens_name.'[i].form,\'submit\',handle_'.$form_tokens_name.');' : '').'
		}
		dec_'.$form_tokens_name.'();
	}
';

		// Start decoder
		if ( $this->onoff_encode ) $code .= '
	func_'.$form_tokens_name.'(String("'.$this->encode($script_src,$this->key).'"));
';
		// ... or original script
		else $code .= $script_src;

		// If no behavioral analysis
		if ( !$this->onoff_behavior ) $code .= '
	dec_'.$form_tokens_name.'();
';
		// Add events
		if ( $this->onoff_behavior ) $code .= '
	for ( var i=0; i<'.$form_tokens_name.'.length; i++ ) {
		form_token_Event.add('.$form_tokens_name.'[i].form,\'keydown\',handle_'.$form_tokens_name.');
		form_token_Event.add('.$form_tokens_name.'[i].form,\'mousedown\',handle_'.$form_tokens_name.');'
.($this->mobile_browser ? '
		form_token_Event.add('.$form_tokens_name.'[i].form,\'submit\',handle_'.$form_tokens_name.');' : '').'
	}
';
		// End scripts
		$code .= '
// ]]></script>
';

		// Modify input form_token
		$event['s_fields'] = preg_replace('#(<input type="hidden" name="form_token" value=")([0-9a-f]+)(" />)#', '${1}' . $prev . '${2}' . $succ . '$3'. $code, $event['s_fields']);

	}

	public function log_antobot100500($event) {

		global $phpbb_container, $phpbb_log;

		// NOT for adm ot install
		if ( defined('ADMIN_START') || defined('IN_INSTALL') ) return;

		// ON/OFF
		if ( isset($this->config['antibot100500_onoff_antibot']) ) {
			$this->onoff_antibot = $this->config['antibot100500_onoff_antibot'] ? true : false;
		}
		// Switched OFF ?
		if ( !$this->onoff_antibot ) return;

		// Options
		if ( isset($this->config['antibot100500_onoff_log_users']) ) {
			$this->onoff_log_users = $this->config['antibot100500_onoff_log_users'] ? true : false;
		}
		// Log swithed OFF ?
		if ( !$this->onoff_log_users ) return;

		// rootref context
		$context = $phpbb_container->get('template_context');
		$rootref = &$context->get_root_ref();
		
		// Form invalid detected...
		if ( isset($rootref["ERROR"])
		  && !empty($rootref["ERROR"])
		  && (strpos($this->user->lang["FORM_INVALID"],$rootref["ERROR"]) !== false)
		   ) {
			$script_name = $rootref["SCRIPT_NAME"];
			$user_browser = $rootref["S_USER_BROWSER"];
			$user_email = '';
			$log_operation = 'LOG_ANTIBOT100500_FORM_INVALID_ON';

			if ( $rootref["S_USER_LOGGED_IN"] ) {
				// ...possible, posting: user logged in
				$user_name = $rootref["S_USERNAME"];
				// ON/OFF for registered users
				if ( isset($this->config['antibot100500_onoff_registered']) ) {
					$this->onoff_registered = $this->config['antibot100500_onoff_registered'] ? true : false;
				}
				// Registered user and OFF for registered users ?
				if ( $this->user->data['is_registered'] && !$this->onoff_registered ) {
					$log_operation = 'LOG_ANTIBOT100500_FORM_INVALID_OFF_USER';
				}
			} else {
				// ...logging and other
				$user_name = !empty($rootref["USERNAME"]) ? $rootref["USERNAME"] : $rotref["S_USERNAME"];
				$user_email = !empty($rootref["EMAIL"]) ? $rootref["EMAIL"] : '';
			}

			$user_name .= '<br />script » '.$script_name;
			$user_name .= '<br />useragent » '.$user_browser;
			if ( !empty($user_email) )
			$user_name .= '<br />e-mail » '.$user_email;

			$user_ip = (empty($this->user->ip)) ? '' : $this->user->ip;

			$phpbb_log->add('user', $this->user->data['user_id'], $user_ip, $log_operation, time(), array($user_name));

		}
	}
	
	/**
	 * Add permissions
	 *
	 * @param object $event The event object
	 * @return null
	 * @access public
	 */
	public function add_permission($event)
	{
		$permissions = $event['permissions'];
		$permissions['a_antibot100500'] = array('lang' => 'ACL_A_ANTIBOT100500', 'cat' => 'misc');
		$event['permissions'] = $permissions;
	}
}
